==> spread-5.3.0.5-py3-none-any.whl: spread_mercury/responses.py <==
import datetime
import struct

from .errors import WriteError, BadConnectResponse


class MResponse:
    _addr = None
    _data = None
    value = None

    def __init__(self, cmd, frame, *params):
        self.cmd = cmd
        self._addr = int.from_bytes(frame[:1], 'little')
        self._data = frame[1:]

        self.parse(*params)

    def parse(self, *params):
        pass

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, self.value)


class MercuryErrorResponse(MResponse):
    value = None


class WriteResponse(MResponse):
    result = None

    def parse(self, *params):
        result = self._data[0]
        self.result = result
        mess = 'Неизвестная ошибка записи'
        if result != 0x0:
            if result == 1:
                mess = 'Недопустимая команда или параметр'
            elif result == 2:
                mess = 'Внутренняя ошибка счётчика'
            elif result == 3:
                mess = 'Недостаточен уровень для удовлетворения запроса'
            elif result == 4:
                mess = 'Внутренние часы счётчика уже корректировались в течение текущих суток'
            elif result == 5:
                mess = 'Не открыт канал связи'
            raise WriteError(self.cmd, result, mess)

    @property
    def success(self):
        return self.result == 0x0

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, 'OK' if self.result == 0x0 else 'Error {}'.format(self.result))


class ConnectSuccess(MResponse):
    def parse(self, *params):
        if len(self._data) != 1:
            raise BadConnectResponse()

        self.value = self._data[0] == 0x00


class SerialResponse(MResponse):
    _serial = None
    _date = None

    def parse(self, *params):
        self._serial = ' '.join(str(b).rjust(2, '0') for b in self._data[:4])
        self._date = datetime.date(2000+self._data[-1], self._data[-2], self._data[-3])
        self.value = dict(serial=self._serial, date=str(self._date).replace(' ', 'T'))


class DateTimeResponse(MResponse):
    time1 = None
    time2 = None

    def parse(self, *params):
        arr = []
        for ch in self._data[-7::-1]:
            arr.append(int(hex(ch)[2:]))
        if arr[0] != 0:
            arr[0] += 2000
            self.time1 = datetime.datetime(*arr)

        arr = []
        for ch in self._data[-1:-7:-1]:
            arr.append(int(hex(ch)[2:]))
        if arr[0] != 0:
            arr[0] += 2000
            self.time2 = datetime.datetime(*arr)

        self.value = dict(time1=str(self.time1).replace(' ', 'T'), time2=str(self.time2).replace(' ', 'T'))


class DateTimeDaySeasonResponse(MResponse):
    time = None
    weekday = None
    is_winter = None

    def parse(self, *params):
        d = self._data

        self.weekday = int(hex(d[3])[2:])
        self.is_winter = int(hex(d[7])[2:]) == 1
        self.time = datetime.datetime(second=int(hex(d[0])[2:]),
                                      minute=int(hex(d[1])[2:]),
                                      hour=int(hex(d[2])[2:]),
                                      day=int(hex(d[4])[2:]),
                                      month=int(hex(d[5])[2:]),
                                      year=2000 + int(hex(d[6])[2:]))

        self.value = dict(time=str(self.time).replace(' ', 'T'), weekday=self.weekday, dst=self.is_winter)


class StoredEnergyResponse(MResponse):
    nrp = None
    nrm = None
    nap = None
    nam = None

    def parse(self, *params):
        d = self._data
        self.nap = struct.unpack('>i', b'' + d[1::-1]+d[3:1:-1])[0]
        self.nap = 0 if self.nap == -1 else self.nap / params[0]
        self.nam = struct.unpack('>i', b'' + d[5:3:-1]+d[7:5:-1])[0]
        self.nam = 0 if self.nam == -1 else self.nam / params[0]
        self.nrp = struct.unpack('>i', b'' + d[9:7:-1]+d[11:9:-1])[0]
        self.nrp = 0 if self.nrp == -1 else self.nrp / params[0]
        self.nrm = struct.unpack('>i', b'' + d[13:11:-1]+d[15:13:-1])[0]
        self.nrm = 0 if self.nrm == -1 else self.nrm / params[0]

        self.value = dict(active=self.nap, active_reverse=self.nam, reactive=self.nrp, reactive_reverse=self.nrm)

    def __str__(self):
        return '[{}] A+: {:.3f} | A-: {:.3f} | R+: {:.3f} | R-: {:.3f}'\
            .format(self.cmd.__class__.__name__, self.nap, self.nam, self.nrp, self.nrm)


class ValueResponse(MResponse):
    def parse(self, *params):
        d = self._data.rjust(3, b'\x00')
        self.value = struct.unpack('>i', b'\x00' + d[0:1] + d[2:] + d[1:2])[0] / params[0]

    def __str__(self):
        return '[{}] phase:{}; value: {}'.format(self.cmd.__class__.__name__, self.cmd.phase, self.value)


class ValueResponsePower(ValueResponse):
    def parse(self, *params):
        self._data = int(bin(self._data[0]).rjust(5, '0')[4:], 2).to_bytes(1, 'little') + self._data[1:]
        super(ValueResponsePower, self).parse(*params)


class LoadStateResponse(MResponse):
    MODES = ['Telemetry', 'LoadControl']

    def parse(self, *params):
        self.value = dict(
            PulseMode=self.MODES[   self._data[0] & 0x01],
            Status=dict(
                power=              self._data[0] & 0x02 != 0,
                energy=             self._data[0] & 0x04 != 0,
                interface=          self._data[0] & 0x08 == 0,
                tariff1=            self._data[0] & 0x10 == 0,
                tariff2=            self._data[0] & 0x20 == 0,
                tariff3=            self._data[0] & 0x40 == 0,
                tariff4=            self._data[0] & 0x80 == 0,

                load=               self._data[1] & 0x40 == 0),
            LoadOn=                 self._data[1] & 0x02 == 0
        )
        return

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, ' '.join([bin(b)[2:].rjust(8, '0') for b in self._data]))


class FeaturesResponse(MResponse):
    hexed_password = False

    def parse(self, *params):
        # 30: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 58: [Features] 10110110 11100011 11000011 10010001 00000111 00000000

        # 78: [Features]
        # 93: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 30: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 26: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 28: [Features] 10110100 11100011 11000010 10010111 11011111 01001000

        # self.hexed_password = self._data[5] & 0x20 != 0
        pass

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, ' '.join([bin(b)[2:].rjust(8, '0') for b in self._data]))


class VersionResponse(MResponse):
    def parse(self, *params): self.value = '{}.{}.{}'.format(*self._data[:3])


class TransformRateResponse(MResponse):
    rv = None
    ra = None

    def parse(self, *params):
        self.rv, self.ra = struct.unpack('>hh', self._data)
        self.value = dict(RateVoltage=self.rv, RateAmperage=self.ra)

==> spread-5.7.0.72-py3-none-any.whl: spread_mercury/responses.py <==
import datetime
import struct

from .errors import WriteError, BadConnectResponse


class MResponse:
    _addr = None
    _data = None
    value = None

    def __init__(self, cmd, frame, *params):
        self.cmd = cmd
        self._addr = frame[0]
        self._data = frame[1:]

        self.parse(*params)

    def parse(self, *params):
        pass

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, self.value)


class MercuryErrorResponse(MResponse):
    value = None
    
    def __init__(self, cmd):
        super(MercuryErrorResponse, self).__init__(cmd, cmd.pack[:1])


class WriteResponse(MResponse):
    result = None

    def parse(self, *params):
        result = self._data[0]
        self.result = result
        mess = 'Неизвестная ошибка записи'
        if result != 0x0:
            if result == 1:
                mess = 'Недопустимая команда или параметр'
            elif result == 2:
                mess = 'Внутренняя ошибка счётчика'
            elif result == 3:
                mess = 'Недостаточный уровень доступа для удовлетворения запроса'
            elif result == 4:
                mess = 'Внутренние часы счётчика уже корректировались в течение текущих суток'
            elif result == 5:
                mess = 'Не открыт канал связи'
            raise WriteError(self.cmd, result, mess)

    @property
    def success(self):
        return self.result == 0x0

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, 'OK' if self.success else 'Error {}'.format(self.result))


class ConnectSuccess(MResponse):
    def parse(self, *params):
        if len(self._data) != 1:
            raise BadConnectResponse()

        self.value = self._data[0] == 0x00


class SerialResponse(MResponse):
    _serial = None
    _date = None

    def parse(self, *params):
        self._serial = ' '.join(str(b).rjust(2, '0') for b in self._data[:4])
        self._date = datetime.date(2000+self._data[-1], self._data[-2], self._data[-3])
        self.value = dict(serial=self._serial, date=str(self._date).replace(' ', 'T'))


class DateTimeResponse(MResponse):
    time1 = None
    time2 = None

    def parse(self, *params):
        arr = []
        for ch in self._data[-7::-1]:
            arr.append(int(hex(ch)[2:]))
        if arr[0] != 0:
            arr[0] += 2000
            self.time1 = datetime.datetime(*arr)

        arr = []
        for ch in self._data[-1:-7:-1]:
            arr.append(int(hex(ch)[2:]))
        if arr[0] != 0:
            arr[0] += 2000
            self.time2 = datetime.datetime(*arr)

        self.value = dict(time1=str(self.time1).replace(' ', 'T'), time2=str(self.time2).replace(' ', 'T'))


class DateTimeDaySeasonResponse(MResponse):
    time = None
    weekday = None
    is_winter = None

    def parse(self, *params):
        d = self._data

        self.weekday = int(hex(d[3])[2:])
        self.is_winter = int(hex(d[7])[2:]) == 1
        self.time = datetime.datetime(second=int(hex(d[0])[2:]),
                                      minute=int(hex(d[1])[2:]),
                                      hour=int(hex(d[2])[2:]),
                                      day=int(hex(d[4])[2:]),
                                      month=int(hex(d[5])[2:]),
                                      year=2000 + int(hex(d[6])[2:]))

        self.value = dict(time=str(self.time).replace(' ', 'T'), weekday=self.weekday, dst=self.is_winter)


class StoredEnergyResponse(MResponse):
    nrp = None
    nrm = None
    nap = None
    nam = None

    def parse(self, *params):
        d = self._data
        self.nap = struct.unpack('>i', b'' + d[1::-1]+d[3:1:-1])[0]
        self.nap = 0 if self.nap == -1 else self.nap / params[0]
        self.nam = struct.unpack('>i', b'' + d[5:3:-1]+d[7:5:-1])[0]
        self.nam = 0 if self.nam == -1 else self.nam / params[0]
        self.nrp = struct.unpack('>i', b'' + d[9:7:-1]+d[11:9:-1])[0]
        self.nrp = 0 if self.nrp == -1 else self.nrp / params[0]
        self.nrm = struct.unpack('>i', b'' + d[13:11:-1]+d[15:13:-1])[0]
        self.nrm = 0 if self.nrm == -1 else self.nrm / params[0]

        self.value = dict(active=self.nap, active_reverse=self.nam, reactive=self.nrp, reactive_reverse=self.nrm)

    def __str__(self):
        return '[{}] A+: {:.3f} | A-: {:.3f} | R+: {:.3f} | R-: {:.3f}'\
            .format(self.cmd.__class__.__name__, self.nap, self.nam, self.nrp, self.nrm)


class ValueResponse(MResponse):
    def parse(self, *params):
        d = self._data.rjust(3, b'\x00')
        self.value = struct.unpack('>i', b'\x00' + d[0:1] + d[2:] + d[1:2])[0] / params[0]

    def __str__(self):
        return '[{}] phase:{}; value: {}'.format(self.cmd.__class__.__name__, self.cmd.phase, self.value)


class ValueResponsePower(ValueResponse):
    def parse(self, *params):
        self._data = (self._data[0] & 0b111111).to_bytes(1, 'little') + self._data[1:]
        super(ValueResponsePower, self).parse(*params)


class LoadStateResponse(MResponse):
    MODES = ['Telemetry', 'LoadControl']

    def parse(self, *params):
        self.value = dict(
            PulseMode=self.MODES[   self._data[0] & 0x01],
            Status=dict(
                power=              self._data[0] & 0x02 != 0,
                energy=             self._data[0] & 0x04 != 0,
                interface=          self._data[0] & 0x08 == 0,
                tariff1=            self._data[0] & 0x10 == 0,
                tariff2=            self._data[0] & 0x20 == 0,
                tariff3=            self._data[0] & 0x40 == 0,
                tariff4=            self._data[0] & 0x80 == 0,

                load=               self._data[1] & 0x40 == 0),
            LoadOn=                 self._data[1] & 0x02 == 0
        )
        return

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, ' '.join([bin(b)[2:].rjust(8, '0') for b in self._data]))


class FeaturesResponse(MResponse):
    hexed_password = False

    def parse(self, *params):
        # 30: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 58: [Features] 10110110 11100011 11000011 10010001 00000111 00000000

        # 78: [Features]
        # 93: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 30: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 26: [Features] 10110100 11100011 11000010 10010111 11011111 01001000
        # 28: [Features] 10110100 11100011 11000010 10010111 11011111 01001000

        # self.hexed_password = self._data[5] & 0x20 != 0
        pass

    def __str__(self):
        return '[{}] {}'.format(self.cmd.__class__.__name__, ' '.join([bin(b)[2:].rjust(8, '0') for b in self._data]))


class VersionResponse(MResponse):
    def parse(self, *params): self.value = '{}.{}.{}'.format(*self._data[:3])


class TransformRateResponse(MResponse):
    rv = None
    ra = None

    def parse(self, *params):
        self.rv, self.ra = struct.unpack('>hh', self._data)
        self.value = dict(RateVoltage=self.rv, RateAmperage=self.ra)
